""" this module is used to write 3D data to BPCH2 file 


  Functions
  -----------------------------------------------------
  1. open_3d_bpch2_file: open one new file to write 
  2. write_3d_field: write 3D data into bpch2 file 
  3. close_3d_bpch2_file: close file 
  

"""
# import module  
from pylab import *  # plot package
from numpy import *  # numeric calculation package
import ESA.util.geo_constant as gc  #  geophysical constants
import ESA.util.time_module as tm      # time format conversion 
import ESA.util.gen_plots as gpl       # interface to map and contour plots
import ESA.util.bpch2_rw_py as bp




    
def open_3d_bpch2_file(flnm, title='flux', do_ext=True):
    """
    Inputs:
    ---------------------------------------------
    1. flnm:<str>: file name 
    2. title: <str>: title of the data
    

    Returns:
    ---------------------------------------------
    1. funit:<int>L file unit 
    
    
    """
    ext1=bp.get_name_ext_2d()
    ext2=bp.get_res_ext()
    funit=38
    if (do_ext):
        full_flnm=flnm.strip()+"."+ext1+"."+ext2
    else:
        full_flnm=flnm.strip()
    
    stat=bp.open_bpch2_for_write(funit,full_flnm, title)
    
    if (stat<>0):
        print 'wrong in open flux '+full_flnm
    
    return funit  

def write_3d_field(funit,data, \
                   lon, lat, \
                   tau0, tau1,\
                   ntracer, \
                   modelname,\
                   category='CO2',\
                   unit="vmr", \
                   do_debug=False):
    
    """
    save 3D fields to one file
    
    Inputs:
    --------------------------------------
    1. funit:<int>: file unit
    2. data:<array, (nx, ny, nz)>: 3D data
    3. lon:<array, (nlon)>: longitude
    4. lat:<array, (nlon)>: latitude
    5. tau0:<float>: starting time 
    6. tau1:<float>: ending time 
    7. ntracer:<int>: tracer numbers
    8. modelname:<str>: modle Name
    9. category:<str>: tracer category 
    10. unit:<str>: unit 
    
    Returns
    ----------------------------------------------
    1. stat:<int>: IO status
    """
    
    
     
        
    lonres=lon[2]-lon[1] # do not us lon[0]-lon[1] as the first one is different from the others 
    latres=lat[2]-lat[1]
    
    
    centre180=1

    halfpolar=bp.get_halfpolar()
    ifirst, jfirst, lfirst=1, 1, 1
    
    dims=shape(data)
    reserved="9999999"
    
    stat = bp.write_bpch2_data(funit,modelname,category,reserved, \
                               lonres,latres,halfpolar,centre180,\
                               ntracer,unit,tau0,tau1,\
                               ifirst,jfirst,lfirst,data)
    return stat

def close_3d_bpch2_file(funit):
    """
    Inputs:
    ============================================
    1. funit:<int>: file unit

    
    """
    bp.close_bpch2_file(funit)


        
    






























    
              
    
        
