#!/usr/bin/env python
"""
An example of how to use wx or wxagg in an application with the Basemap module
"""



from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure

from mpl_toolkits.basemap import Basemap
import numpy as np
from pylab import *
from numpy import *
from wx import *
TIMER_ID = 1200


class CanvasFrame(Frame):
   def __init__(self, parent, ID, title, \
                data, rlon=None, rlat=None,use_pcolor=0,\
                size=wx.DefaultSize, pos=wx.DefaultPosition, \
                style=wx.DEFAULT_DIALOG_STYLE, useMetal=False,\
                val_list=None, val_name_list=None,\
                plot_keywords={'Choice':0}):
      
      # pre = wx.PreDialog()
      # pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
      # pre.Create(parent, ID, title, pos, size, style)
      Frame.__init__(self,parent,ID,
                     'CanvasFrame',size=(550,350))
      self.SetBackgroundColour(NamedColor("WHITE"))
      self.parent=parent
      self.TIMER_ID = TIMER_ID
      self.figure = Figure()
      # self.figure.show()
      
      self.cur_id=-1
      
      self.canvas = FigureCanvas(self, -1, self.figure)
      self.ax=self.figure.add_axes([0.1,0.1,0.85,0.85])
      # self.ax = self.figure.add_subplot(111)
      self.sizer = BoxSizer(VERTICAL)
      self.sizer.Add(self.canvas, 1, LEFT | TOP | GROW)
      self.SetSizer(self.sizer)
      self.Fit()
      self.add_toolbar()  # comment this out for no toolbar
      self.m=None
      self.cs0=None
      
      self.canvas.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
      
      self.plot_map(data, rlon=rlon, rlat=rlat,use_pcolor=use_pcolor,  keywords=plot_keywords)
      self.is_stop=False
      self.val_list=val_list
      self.val_name_list=val_name_list
      self.t=wx.Timer(self, TIMER_ID)

      if (self.val_list<>None):
         self.t.Start(200)
      
      #      self.Bind(wx.EVT_TIMER, self.onTimer, TIMER_ID)
      EVT_TIMER(self, TIMER_ID, self.onTimer)
      self.Show(True)
      
   def add_toolbar(self):
      self.toolbar = NavigationToolbar2Wx(self.canvas)
      self.toolbar.Realize()
      if Platform == '__WXMAC__':
         # Mac platform (OSX 10.3, MacPython) does not seem to cope with
         # having a toolbar in a sizer. This work-around gets the buttons
         # back, but at the expense of having the toolbar at the top
         self.SetToolBar(self.toolbar)
      else:
         # On Windows platform, default window size is incorrect, so set
         # toolbar width to figure width.
          tw, th = self.toolbar.GetSizeTuple()
          fw, fh = self.canvas.GetSizeTuple()
          # By adding toolbar in sizer, we are able to put it at the bottom
          # of the frame - so appearance is closer to GTK version.
          # As noted above, doesn't work for Mac.
          self.toolbar.SetSize(Size(fw, th))
          self.sizer.Add(self.toolbar, 0, LEFT | EXPAND)
          # update the axes menu on the toolbar
      self.toolbar.update()
           
   
   def plot_map(self, data, rlon=None, rlat=None,use_pcolor=0,  keywords={'Choices':0}):
      
      """ display  the data in a map
      keywords: dict which can include the following keywords  minv=0.0, maxv=0.0, dv=0.0,
      show_map=0, map_proj='cyl', show_lat=0, show_lon=0, lat_0=0.0, lon_0=0.0,
      minlat=-90.0, maxlat=90.0, minlon=0.0, maxlon=360.0, use_log=0, level=level
      """
      
    
    
		
      minv=0.0
      if ('minv' in keywords):
         minv=keywords['minv']
         
      maxv=0.0
      
      if ('maxv' in keywords):
         maxv=keywords['maxv']
        
      dv=0.0
      if ('dv' in keywords):
         dv=keywords['dv']
         
         
      if ('dv' in keywords):
         dv=keywords['dv']
        
        
        	
      if (maxv>minv):
         rlvl=arange(minv, maxv+dv, dv)
         rlvl[0]=-999.0
         rlvl[size(rlvl)-1]=999.
        
            
      stitle=""
      add_str=""
      if ('title' in keywords):
         add_str=keywords['title']
         add_str=add_str.strip()
         stitle=stitle+' '+add_str
        
      if ('unit' in keywords):
         add_str=keywords['unit']
         add_str=add_str.strip()
         stitle=stitle+'('+ add_str +')'
            
					
      cbar_vert=1
      if ('cbar_vert' in keywords):
         cbar_vert=keywords['cbar_vert']
         
      if (cbar_vert==1):
         orientation='vertical'
      else:
         orientation='horizontal'
            
         
      show_lat=1
      if ('show_lat' in keywords):
         show_lat=keywords['show_lat']
         
      show_lon=1
      if ('show_lon' in keywords):
         show_lon=keywords['show_lon']
         
      nlon, nlat=shape(data)
            
      vals=array(data)
      
    
      map_proj='cyl'
      if ('map_proj' in keywords):
         map_proj=keywords['map_proj']
         
      lat_0=0.0
      if ('lat_0' in keywords):
         lat_0=keywords['lat_0']
         
      minlat=-90.0
      maxlat=90.0
      
      if ('minlat' in keywords):
         minlat=keywords['minlat']
         
      if ('maxlat' in keywords):
         maxlat=keywords['maxlat']
         
      if (rlat==None):
         dlat=(maxlat-minlat)/nlat
         rlat=arange(minlat,maxlat, dlat)
                        
                        
      minlon=-180
      maxlon=180.0
      
      if ('minlon' in keywords):
         minlon=keywords['minlon']
         
      if ('maxlon' in keywords):
         maxlon=keywords['maxlon']
         
      if (rlon==None):
         dlon=(maxlon-minlon)/nlon
         rlon=arange(minlon, maxlon, dlon)
         
      lon_0=0

      do_bdr=0
      if ('do_bdr' in keywords):
         do_bdr=keywords['do_bdr']
         
         
      if ('lon_0' in keywords):
         lon_0=keywords['lon_0']
         
         boundinglat=45

      if ('boundinglat' in keywords):
         boundinglat=keywords['boundinglat']
         
      if (map_proj=='npstere' or map_proj=='spstere'):
         self.m=Basemap(ax=self.ax, projection=map_proj, lon_0=lon_0, boundinglat=boundinglat)
      elif (map_proj=='ortho'):
         self.m=Basemap(ax=self.ax, projection=map_proj, lon_0=lon_0, lat_0=lat_0)
      else:
         self.m=Basemap(ax=self.ax, llcrnrlon=minlon, llcrnrlat=minlat, \
                  urcrnrlon=maxlon, urcrnrlat=maxlat,projection=map_proj, lon_0=lon_0, lat_0=lat_0)
        
      if (rlon[-1]<rlon[0]+360.0):
         rlon=resize(rlon, nlon+1)
         rlon[-1]=rlon[0]+360.0
         vals=squeeze(vals)
         vals=resize(vals, [nlon+1, nlat])
        
      x,y=self.m(*meshgrid(rlon, rlat))
        
      cmap=cm.Set1
      if ('cmap' in keywords):
         cmap=keywords['cmap']
      print 'maxv--minv in plot map', maxv, minv
      
      if (maxv>minv):
         if (use_pcolor==1):
            #cs0=self.m.pcolor(x, y, transpose(vals),  shading='flat', vmin=minv, vmax=maxv, cmap=cmap)
            cs0=self.m.imshow(transpose(vals), vmin=minv, vmax=maxv)
            
         else:
            # cs0=self.m.contourf(x, y, transpose(vals), rlvl)
            cs0=self.m.imshow(x, y, transpose(vals), vmin=minv, vmax=maxv)
            
      else:
         if (use_pcolor==1):
            # cs0=self.m.pcolor(x, y, transpose(vals),shading='flat', cmap=cmap)
            cs0=self.m.imshow(x, y, transpose(vals))
            
         else:
            # cs0=self.m.contourf(x, y, transpose(vals))
            cs0=self.m.imshow(x, y, transpose(vals), vmin=minv, vmax=maxv)
      
            
      # info(m.drawcoastlines)
      self.m.drawcoastlines(color='k', linewidth=1.5)
      # m.drawcountries(color=white)
      self.m.drawmapboundary()
      if (show_lat==1):
         self.m.drawparallels(arange(minlat, maxlat+30.0, 30.), labels=[1,0, 0,0])
         
      if (show_lon==1):
         self.m.drawmeridians(arange(minlon,maxlon+60,60.),labels=[0,0,0,1])
         # self.ax.title(stitle)
      
      show_colorbar=1
      if ('cb' in keywords):
         show_colorbar=keywords['cb']
      
      if (show_colorbar==1):
         # cax = self.figure.add_axes([0.85,0.1,0.075,0.85])
         self.figure.colorbar(cs0, ax=self.ax,orientation=orientation, fraction=0.10)
         # colorbar(cs0, cax=cax, orientation=orientation)
            # pass
      if (do_bdr==1):
         lvl=arange(max(vals.flat))
         print shape(x), shape(y), shape(vals)
         # cs2=m.contour(x[0:-1,:], y[0:-1,:], transpose(vals), lvl, colors='k', linewidth=0.5)
      self.cs0=cs0
      self.figure.canvas.draw()
      
      # return m
      
   def onTimer(self, evt):
      self.is_stop=False
      
      self.cur_id=self.cur_id+1
      if (self.cur_id==len(self.parent.val_list)):
          self.cur_id=0
      
      print 'Timer', self.cur_id
      
      z=self.val_list[self.cur_id]
      vname=self.val_name_list[self.cur_id]
      self.ax.set_title(vname.strip())
      
      self.cs0.set_array(transpose(z))
      # print max(z.flat)
      
      self.canvas.draw()
        #self.canvas.gui_repaint()  # jdh wxagg_draw calls this already
      # flnm=vname.strip()
      # print 'flnm---', flnm
      # savefig(flnm+'.png')
      
              
   def OnDoubleClick(self, event):
      if (not self.is_stop):
         self.t.Stop()
         self.is_stop=True
      else:
         if (self.val_list<>None):
            self.t.Start(200)
      
   
   def onEraseBackground(self, evt):
        # this is supposed to prevent redraw flicker on some X servers...
        pass
     
   

