"""
Generate zonal mean animate plot in wx backend

"""



from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure

from mpl_toolkits.basemap import Basemap
import numpy as np
from pylab import *
from numpy import *
from wx import *
TIMER_ID = 1200


class CanvasFrame(Frame):
   def __init__(self, parent, ID, title, \
                data, rlat=None, rpres=None,use_pcolor=0,\
                   size=wx.DefaultSize, pos=wx.DefaultPosition, \
                   style=wx.DEFAULT_DIALOG_STYLE, useMetal=False,\
                   val_list=None, val_name_list=None,\
                   plot_keywords={'Choice':0}):
      
      # pre = wx.PreDialog()
      # pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
      # pre.Create(parent, ID, title, pos, size, style)
      Frame.__init__(self,parent,ID,
                     'CanvasFrame',size=(550,350))
      self.SetBackgroundColour(NamedColor("WHITE"))
      self.parent=parent
      self.TIMER_ID = TIMER_ID
      self.figure = Figure()
      # self.figure.show()
      
      self.cur_id=-1
      
      self.canvas = FigureCanvas(self, -1, self.figure)
      self.ax=self.figure.add_axes([0.12,0.12,0.82,0.82])
      
                       
      # self.ax = self.figure.add_subplot(111)
      self.sizer = BoxSizer(VERTICAL)
      self.sizer.Add(self.canvas, 1, LEFT | TOP | GROW)
      self.SetSizer(self.sizer)
      self.Fit()
      self.add_toolbar()  # comment this out for no toolbar
      self.m=None
      self.cs0=None
      self.nx, self.nlvl=shape(data)
      self.canvas.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
      
      self.plot_zonal_mean(data, rlat=rlat, rpres=rpres,use_pcolor=use_pcolor,  keywords=plot_keywords)
      
      # self.ax.set_yscale('log')
      # self.ax.set_ylim([max(rpres), min(rpres)])
      zylim=self.ax.get_ylim()
      zylim=zylim[::-1]
      self.ax.set_ylim(zylim)
      if ('yticks' in plot_keywords):
         ylbl=plot_keywords['yticks']
         if (rpres[-1]>rpres[0]):
            yx=searchsorted(rpres, ylbl)
         else:
            
            yx=searchsorted(rpres[::-1], ylbl)
            yx=self.nlvl-yx-1
            yx=where(yx<0, 0, yx)
         
         
         self.ax.yaxis.set_ticks(yx)
         self.ax.yaxis.set_ticklabels(ylbl)
      
      else:
         yx=self.ax.get_yticks()
         yx=yx-yx[0]
         yx=yx.astype(int)
         
      
         idx=np.where(yx<np.size(rpres))
         yx=yx[idx]

         ylbl=rpres[yx]
         ylbl=ylbl+0.5
         
         ylbl=ylbl.astype(int)
   
         self.ax.yaxis.set_ticks(yx)
         self.ax.yaxis.set_ticklabels(ylbl)
      
   
      if ('xticks' in plot_keywords):
         xlbl=plot_keywords['xticks']
         xx=searchsorted(rlat, xlbl)
         
         self.ax.xaxis.set_ticks(xx)
         self.ax.xaxis.set_ticklabels(xlbl)
      
      else:
   
         xx=self.ax.get_xticks()
         xx=xx-xx[0]
         xx=xx.astype(int)
         idx=np.where(xx<np.size(rlat))
         xx=xx[idx]
         xlbl=rlat[xx]
         self.ax.xaxis.set_ticks(xx)
         self.ax.xaxis.set_ticklabels(xlbl)
      
      
      
      
      self.rlat=rlat
      self.rpres=rpres
      self.keywords=plot_keywords
      self.use_pcolor=use_pcolor
      
      self.is_stop=False
      self.val_list=val_list
      self.val_name_list=val_name_list
      self.t=wx.Timer(self, TIMER_ID)

      if (self.val_list<>None):
         self.t.Start(200)
      
      #      self.Bind(wx.EVT_TIMER, self.onTimer, TIMER_ID)
      EVT_TIMER(self, TIMER_ID, self.onTimer)
      self.Show(True)
      
   def add_toolbar(self):
      self.toolbar = NavigationToolbar2Wx(self.canvas)
      self.toolbar.Realize()
      if Platform == '__WXMAC__':
         # Mac platform (OSX 10.3, MacPython) does not seem to cope with
         # having a toolbar in a sizer. This work-around gets the buttons
         # back, but at the expense of having the toolbar at the top
         self.SetToolBar(self.toolbar)
      else:
         # On Windows platform, default window size is incorrect, so set
         # toolbar width to figure width.
          tw, th = self.toolbar.GetSizeTuple()
          fw, fh = self.canvas.GetSizeTuple()
          # By adding toolbar in sizer, we are able to put it at the bottom
          # of the frame - so appearance is closer to GTK version.
          # As noted above, doesn't work for Mac.
          self.toolbar.SetSize(Size(fw, th))
          self.sizer.Add(self.toolbar, 0, LEFT | EXPAND)
          # update the axes menu on the toolbar
      self.toolbar.update()
           
   
   def plot_zonal_mean(self, data, rlat=None, rpres=None,use_pcolor=0,  keywords={'Choices':0}):
      
      """ display  the data in a map
      keywords: dict which can include the following keywords  minv=0.0, maxv=0.0, dv=0.0,
      show_map=0, map_proj='cyl', show_lat=0, show_lon=0, lat_0=0.0, lon_0=0.0,
      minlat=-90.0, maxlat=90.0, minlon=0.0, maxlon=360.0, use_log=0, level=level
      """
      
    
      if (rlat==None):
         rlat=self.rlat
      
      if (rpres==None):
         rpres=self.rpres
      
      
		
      minv=0.0
      if ('minv' in keywords):
         minv=keywords['minv']
         
      maxv=0.0
      
      if ('maxv' in keywords):
         maxv=keywords['maxv']
        
      dv=0.0
      if ('dv' in keywords):
         dv=keywords['dv']
         
         
      if ('dv' in keywords):
         dv=keywords['dv']
        
        
      
      if (maxv>minv):
         rlvl=arange(minv, maxv+dv, dv)
         # rlvl[0]=-999.0
         # rlvl[size(rlvl)-1]=999.
      else:

         maxv=max(vals.flat)
         minv=min(vals.flat)
         
         if (minv==-999.0):
            vals=where(vals==-999.0, maxv+100, vals)
            minv=min(vals)
            vals=where(vals==maxv+100, minv, vals)
         
         
         dv=(maxv-minv)/10.0
         
            
      stitle=""
      add_str=""
      if ('title' in keywords):
         add_str=keywords['title']
         add_str=add_str.strip()
         stitle=stitle+' '+add_str
        
      if ('unit' in keywords):
         add_str=keywords['unit']
         add_str=add_str.strip()
         stitle=stitle+'('+ add_str +')'
            
      cb=1
      if ('cb' in keywords):
         cb=keywords['cb']
      
      cbar_vert=1
      if ('cbar_vert' in keywords):
         cbar_vert=keywords['cbar_vert']
         
      if (cbar_vert==1):
         orientation='vertical'
      else:
         orientation='horizontal'
            
         
      nlat, nlvl=shape(data)
            
      vals=array(data)
      
      x=rlat
      y=rpres
      cmap=cm.jet
      if (maxv>minv):
      
         if (use_pcolor==1):
            # cs0=self.ax.pcolor(x, y, transpose(vals),  shading='flat', vmin=minv, vmax=maxv, cmap=cmap)
            cs0=self.ax.imshow(transpose(vals),  vmin=minv, vmax=maxv, cmap=cmap)
            
         else:
            cs0=self.ax.imshow(transpose(vals),  vmin=minv, vmax=maxv, cmap=cmap)
            
            # cs0=self.ax.contourf(x, y, transpose(vals), rlvl)
      else:
         
         if (use_pcolor==1):
            # cs0=self.ax.pcolor(x, y, transpose(vals),  shading='flat')
            cs0=self.ax.imshow(transpose(vals), vmin=minv, vmax=maxv, cmap=cmap)
            
         else:
            cs0=self.ax.imshow(transpose(vals), vmin=minv, vmax=maxv, cmap=cmap)
            
            # cs0=self.ax.contourf(x, y, transpose(vals), rlvl)
      
      if (cb==1):
         self.figure.colorbar(cs0, ax=self.ax,orientation=orientation, fraction=0.10)
      
      self.cs0=cs0
      self.figure.canvas.draw()
         
      # return m
      
   def onTimer(self, evt):
      self.is_stop=False
      
      self.cur_id=self.cur_id+1
      if (self.cur_id==len(self.parent.val_list)):
          self.cur_id=0
      
      print 'Timer', self.cur_id
      
      z=self.val_list[self.cur_id]
      vname=self.val_name_list[self.cur_id]
      # self.plot_zonal_mean(z, rlat=self.rlat, rpres=self.rpres,use_pcolor=self.use_pcolor,  keywords=self.keywords)
      self.ax.set_title(vname.strip())
      
      self.cs0.set_array(transpose(z))
      # print max(z.flat), min(z.flat)
      
      
      self.figure.canvas.draw()
      # flnm=vname.strip()
      # print 'flnm---', flnm
      # savefig(flnm+'.png')
      # self.figure.draw()
      
              
   def OnDoubleClick(self, event):
      if (not self.is_stop):
         self.t.Stop()
         self.is_stop=True
      else:
         if (self.val_list<>None):
            self.t.Start(200)
      
   
   def onEraseBackground(self, evt):
        # this is supposed to prevent redraw flicker on some X servers...
        pass
     
   

